@@ -1,5 +1,5 @@
 #
-#	Copyright Saul Youssef, July, 2004
+#	Copyright Saul Youssef, January, 2005
 #
 import string,os
 from types                          import *
@@ -75,12 +75,17 @@ from CommandLineSwitch              import *
 
 from AtomUtils                      import *
 #import Package,Installation,Cat
-import Package,Cat,Already,Watch,SaveRestore
+import Package,Cat,Already,Watch,SaveRestore,Md5sumCheck
 
 from AtomDoc import *
 
 catd = AtomDoc('cat','path')
 
+def md5check(path,md5string):
+	strArg(path,'md5check')
+	strArg(md5string,'md5check')
+	return Md5sumCheck.Md5sumCheck(path,md5string)
+
 def restore(filename):
 	strArg(filename,'restore')
 	return SaveRestore.RestoreFromUninstall(filename)
@@ -476,6 +481,58 @@ def tagGT(arg):
 	if type(arg)==StringType: text = arg
 	else:                     text = `arg`
 	return TagGT(text)
+	
+def patch(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	strArg(text,'patch')
+	return Patch(text)
+	
+def patchLE(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	return PatchLE(text)
+	
+def patchLT(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	return PatchLT(text)
+	
+def patchGE(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	return PatchGE(text)
+	
+def patchGT(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	return PatchGT(text)
+	
+def option(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	strArg(text,'option')
+	return Option(text)
+	
+def optionLE(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	return OptionLE(text)
+	
+def optionLT(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	return OptionLT(text)
+	
+def optionGE(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	return OptionGE(text)
+	
+def optionGT(arg):
+	if type(arg)==StringType: text = arg
+	else:                     text = `arg`
+	return OptionGT(text)
 
 def gccVersion(arg):
 	strArg(arg,'gccVersion')
@@ -657,6 +714,10 @@ def exists(filename):
 	strArg(filename,'exists')
 	return FileExists(filename)
 	
+def askUntilFileExists(path):
+	strArg(path,'askUntilFileExists')
+	return AskUntilFileExists(path)
+	
 def watch(path):
 	strArg(path,'exists')
 	return Watch.Watch(path)
@@ -933,15 +994,23 @@ def download(url):
 	strArg(url,'download')
 	return Download(url)
 	
+def downloaduntarzipwrap(url,env):
+	if tail(url,'.tar.gz') or tail(url,'.tgz') or tail(url,'.tar.Z') or tail(url,'.tar.z'):
+		D = DownloadUntarzip(url,env)
+	else:
+		D = Download(url)
+	return D
+
 def downloadUntar(url,env=''):
 	strArg(url,'downloadUntar')
 	strArg(env,'downloadUntar')
-	return DownloadUntarzip(url,env)
+	return downloaduntarzipwrap(url,env)
 	
 def downloadUntarzip(url,env=''):
 	strArg(url,'downloadUntarzip')
 	strArg(env,'downloadUntarzip')
-	return DownloadUntarzip(url,env)
+	return downloaduntarzipwrap(url,env)
+#	return DownloadUntarzip(url,env)
 
 #	head,tail = os.path.split(url)
 #	d1 = AND(FileExistsOnce(os.path.join('../downloads',tail)),MV(os.path.join('../downloads',tail),tail))
