@@ -14,10 +14,9 @@ class CommandLineSwitch(Environment):
 	def str(self): return self._switch
 	def equal(self,x): return self._switch==x._switch
 	
-	def satisfied(self): 
-		return Reason('Command line switch ['+self._switch+'] not used.',not switch(self._switch))
-	def satisfiable(self): return self.satisfied()
+	def satisfied   (self): return Reason('Command line switch ['+self._switch+'] not used.',not switch(self._switch))
+	def satisfiable (self): return self.satisfied()
 	
-	def acquire(self): return self.satisfied()
-	def retract(self): return Reason()
+	def acquire     (self): return self.satisfied()
+	def retract     (self): return Reason()
 	
