@@ -1,5 +1,5 @@
 #
-#	Copyright, 2004, Saul Youssef
+#	Copyright, 2005, Saul Youssef
 #
 from Base        import *
 from Environment import *
@@ -94,7 +94,8 @@ class Spec(PreOrder,HtmlOut):
 		self.file = ''
 		if tail(self.name,'.pacman'):
 			self.file = self.name
-			self.name = self.name[:-7]
+#			self.name = self.name[:-7]
+			self.name = '' # self.name[:-7]
 		if specstring=='.': self.name,self.cacheName = '','.'
 		self.mayBeModifiedBy = ['']
 			
@@ -118,7 +119,8 @@ class Spec(PreOrder,HtmlOut):
 			q = p._spec<=self
 			if q:
 				if not self.guard=='':
-					r,guard = Source.Source(self.guard).comp()
+#					r,guard = Source.Source(self.guard).comp()
+					r,guard = Source.Source(self.guard,Spec()).comp()
 					if r.ok():
 						if not p._environ.satisfies(guard):
 							r = Reason("Package specification ["+self.str()+"] fails |... requirements of ["+p._spec.str()+"].")
@@ -248,6 +250,17 @@ class Package(Environment,Update,Verify):
 			update      =     self.updateWaiting().ok()
 			loc         =     self.location()
 		return installable,installed,update,loc
+	def getEs(self,typename):
+		es = []
+		def f(E):
+			if E.type=='AND' or E.type=='OR':
+				for e in E: f(e)
+			elif E.type=='lazy package': 
+				pass
+			else:
+				if E.type==typename: es.append(E)
+		f(self._environ)
+		return es
 	def display(self,indent=0,tail=''): 
 		installable,installed,update,loc = self.status()
 
@@ -298,17 +311,30 @@ class Package(Environment,Update,Verify):
 			for e in self._environ: 
 				if e.type=='version' or e.type=='version tuple': 
 					print indent*' '+'           '+`e`
-					break
 		if displayMode('release'):
 			for e in self._environ: 
 				if e.type=='release': 
 					print indent*' '+'           '+`e`
-					break
+		if displayMode('description'):
+			for e in self._environ:
+				if e.type=='description':
+					print indent*' '+'           '+`e`
+		if displayMode('url'):
+			for e in self._environ:
+				if e.type=='url':
+					print indent*' '+'           '+`e`
 		if displayMode('tag'):
 			for e in self._environ: 
 				if e.type=='tag': 
 					print indent*' '+'           '+`e`
-					break
+		if displayMode('patch'):
+			for e in self._environ:
+				if e.type=='patch':
+					print indent*' '+'           '+`e`
+		if displayMode('option'):
+			for e in self._environ:
+				if e.type=='option':
+					print indent*' '+'           '+`e`
 		if   displayMode('src'):
 			for line in       self._sourceCode: print indent*' '+'         [src] '+line[:-1]
 		if displayMode('cmp'):
@@ -360,20 +386,37 @@ class Package(Environment,Update,Verify):
 		self._lastSatisfiable = self._environ.fetch()
 		return self.reasonFixup(self._lastSatisfiable,'installable')
 	def satisfy(self):
-		if self._neutered:
-			r = self.setup()
-			if not r.ok():
-				r.display()
-				r = Reason("Package ["+self.str()+"] not installed at remote installation.")
-				self.lastfail = 1
+		verbo.log('pac','About to begin installing ['+self._spec.str()+']...')
+		r = ask.re('pac','OK to begin installing ['+self._spec.str()+']?')
+		if r.ok():
+			if self._neutered:
+				r = self.setup()
+				if not r.ok():
+					r.display()
+					r = Reason("Package ["+self.str()+"] not installed at remote installation.")
+					self.lastfail = 1
+				else:
+					self.lastsat  = 1
 			else:
-				self.lastsat  = 1
+				r = self._environ.satisfy()
+				if r.ok(): self.lastsat  = 1
+				else:      self.lastfail = 1
+				
+				if r.ok(): 
+					verbo.log('pac','Package ['+self._spec.str()+'] successfully installed...')
+					r = ask.re('pac','Package ['+self._spec.str()+'] has been successfully installed. Keep going?')
+				else:      
+					verbo.log('pac','Package ['+self._spec.str()+'] installation attempt has failed...')
+		return self.reasonFixup(r)
+	def setup(self): 
+		idd = self._id()
+		if not _setupBody.has_key(idd):
+			debug.log('setup','Package ['+self._spec.str()+'] is being setup...')
+			r = self._environ.setup()
+			_setupBody[idd] = ''
 		else:
-			r = self._environ.satisfy()
-			if r.ok(): self.lastsat  = 1
-			else:      self.lastfail = 1
+			r = Reason()
 		return self.reasonFixup(r)
-	def setup(self): return self.reasonFixup(self._environ.setup())
 	def restore(self):
 		r = Reason()
 		if not self._neutered:
@@ -390,16 +433,26 @@ class Package(Environment,Update,Verify):
 #-- Install
 	def uninstall(self,depth=0):
 		r = Reason()
-		if self._neutered or depth<0:
-			pass
-		elif len(self._modifiers)>0:
-			r = allReason(self._modifiers,lambda spec: LazyPackage(spec).uninstall())
-		else:
-			r = self.uninstallBody(depth)
-			self.uninstallSet()
+		if verbo('pac'): print 'About to begin uninstalling ['+self._spec.str()+']...'
+		r = ask.re('pac','OK to begin uninstalling ['+self._spec.str()+']?')
+		if r.ok():
+			if self._neutered or depth<0:
+				pass
+			elif len(self._modifiers)>0:
+				r = allReason(self._modifiers,lambda spec: LazyPackage(spec).uninstall())
+			else:
+				if self.lastsat and len(self.getEs('uninstall shell command'))>0: self.setup()
+				r = self.uninstallBody(depth)
+				if not switch('update'): self.uninstallSet()
 			
-		self.lastsat  = 0
-		self.lastfail = 0
+			self.lastsat  = 0
+			self.lastfail = 0
+			
+			if r.ok(): 
+				if verbo('pac'): print 'Package ['+self._spec.str()+'] successfully uninstalled...'
+				r = ask.re('pac','Package ['+self._spec.str()+'] has beed successfully uninstalled.  Keep going?')
+			else:      
+				verbo.log('pac','Package ['+self._spec.str()+'] uninstallation attempt has failed...')
 		return self.reasonFixup(r,'uninstall')
 	def uninstallBody(self,depth=0):
 		def f(e):
@@ -422,10 +475,12 @@ class Package(Environment,Update,Verify):
 		r = allReasonQB(self.parents(),lambda spec: LazyPackage(spec).uninstallSet())
 		return r
 	def remove(self,depth=0):
-		r = self.uninstall(0)
-		if r.ok(): r = self.desave()
-		if r.ok(): r = self.descend(depth)
-		if r.ok(): r = UniversalCache.UniversalCache('home').remove(self._spec)
+		r = Reason()
+		if depth>=0:
+			r = self.uninstall(0)
+			if r.ok(): r = self.desave()
+			if r.ok(): r = self.descend(depth)
+			if r.ok(): r = UniversalCache.UniversalCache('home').remove(self._spec)
 		return self.reasonFixup(r,'remove')
 	def desave(self):
 		def f(E):
@@ -439,12 +494,12 @@ class Package(Environment,Update,Verify):
 	def descend(self,depth=0):
 		def f(E,depth):
 			r = Reason()
-			if   E.type=='AND' or E.type=='OR':
+			if E.type=='AND' or E.type=='OR':   
 				r = allReason(E,lambda e: f(e,depth))
 			elif E.type=='lazy package':
 				r = E.removeParent(self._spec,depth)
 			return r
-		r = f(self._environ,depth-1)
+		r = f(self._environ,depth)
 		return r
 #-- Update
 	def updateCheck(self):
@@ -468,25 +523,32 @@ class Package(Environment,Update,Verify):
 		return self._environ.updateRemove()
 	def updateWaiting(self): return Reason('No update waiting.',self._update == OR())
 	def updateSelf(self):
-		r = Reason("Can't update relative package ["+self.str()+"].",self._neutered)
+#		verbo.log('up','About to update ['+self._spec.str()+']...')
+#		r = ask.re('up','OK to update ['+self._spec.str()+']?')
+		r = Reason()
 		if r.ok():
-			if not self._update==OR():
-				upd = self.status()[1]
-				if upd:
-					r = self.setup()
-					if r.ok(): 
-						r = LazyPackage(self._spec).uninstall(0)
-				if r.ok():
-					verbo.log('up','Updating ['+self._spec.name+']...')
-					self._environ    = copy.deepcopy(self._update)
-					self._source     = copy.deepcopy(self._update)
-					self._sourceCode = self._updateSourceCode
-					self._update = OR()
-					if upd: r = self.satisfy()
+			r = Reason("Can't update relative package ["+self.str()+"].",self._neutered)
+			if r.ok():
+				if not self._update==OR():
+					upd = self.status()[1]
+					if upd:
+						r = self.setup()
+						if r.ok(): r = LazyPackage(self._spec).uninstall(0)
+					if r.ok():
+						r = ask.re('up','OK to update ['+self._spec.str()+']?')
+						if r.ok():
+							verbo.log('up','Updating ['+self._spec.name+']...')
+							self._environ    = copy.deepcopy(self._update)
+							self._source     = copy.deepcopy(self._update)
+							self._sourceCode = self._updateSourceCode
+							self._update = OR()
+							if upd: r = self.satisfy()
+					else:
+						verbo.log('up',"Can't uninstall ["+self._spec.name+"].  Not updated...")
 				else:
-					verbo.log('up',"Can't uninstall ["+self._spec.name+"].  Not updated...")
-			else:
-				r = self._environ.update()
+					r = self._environ.update()
+#					if r.ok(): verbo.log('up','Package ['+self._spec.str()+'] successfully updated...')
+#					else:      verbo.log('up','Package ['+self._spec.str()+'] update attempt has failed...')
 		return r
 	def update(self):
 		r = Reason("Can't update relative package ["+self.str()+"].",self._neutered)
@@ -521,6 +583,7 @@ _verify        = {}
 _repair        = {}
 
 _setup         = {}
+_setupBody     = {}
 _shellOut      = {}
 
 _pardb         = {}
@@ -535,9 +598,8 @@ class LazyPackage(Package):
 		self._spec      = spec
 		self._modified  = 0
 		self._parent    = Spec()
-		
 		self._lazy = 1
-		
+
 	def lazyApplyU(self,f=lambda p: Reason()):
 		c = UniversalCache.UniversalCache('home')
 		r,p = c.get(self._spec)
@@ -548,6 +610,17 @@ class LazyPackage(Package):
 		else:
 			p = Package()
 		return r,p
+	def lazyApplyV(self,f=lambda p: Reason()):
+		c = UniversalCache.UniversalCache('home')
+		r,p = c.get(self._spec)
+#		if not r.ok(): r,p = self._spec.get()
+		if r.ok():
+			self.modifyApply(p)
+			if r.ok(): r = f(p)
+		else:
+			r = Reason()
+			p = Package()
+		return r,p
 	def lazyApplyM(self,f=lambda p: Reason()):
 		r,p = self.lazyApplyU(f)
 		if not p==Package(): 
@@ -574,9 +647,10 @@ class LazyPackage(Package):
 			p._parents   = filter(lambda sp: not spec<=sp, p._parents  )
 			p._modifiers = filter(lambda sp: not spec<=sp, p._modifiers)
 			UniversalCache.UniversalCache('home').put(p)
-			if p._parents==[] and p._modifiers==[]: r = p.remove(depth)
+			if p._parents==[] and p._modifiers==[]: r = p.remove(depth-1)
 			return r
-		return self.lazyApplyU(f)[0]
+		return self.lazyApplyV(f)[0]
+#		return self.lazyApplyU(f)[0]
 #-- Set
 	def equal(self,p): return self._spec==p._spec
 	def str(self): return self._spec.str()
@@ -608,20 +682,20 @@ class LazyPackage(Package):
 		return db[idd]
 		
 #-- Environment
-	def satisfiable  (self): return self.check(_satisfiable,lambda: self.lazyApplyM(lambda x: x.satisfiable   ())    )
-	def fetch        (self): return self.check(_fetch,      lambda: self.lazyApplyM(lambda x: x.fetch         ())    )
+	def satisfiable  (self): return self.check(_satisfiable,lambda: self.lazyApplyM(lambda x: x.satisfiable      ())    )
+	def fetch        (self): return self.check(_fetch,      lambda: self.lazyApplyM(lambda x: x.fetch            ())    )
 	def satisfied    (self): 
-		r = self.check(_satisfied,  lambda: self.lazyApplyU(lambda x: x.satisfied      ())[0] )
+		r = self.check(_satisfied,  lambda: self.lazyApplyU(lambda x: x.satisfied                            ())[0] )
 		if r.ok(): self.lastsat  = 1
 		else:      self.lastfail = 1
 		return r
 	def satisfy      (self): 
-		r = self.check(_satisfy,    lambda: self.lazyApplyM(lambda x: x.satisfy        ())    )
+		r = self.check(_satisfy,    lambda: self.lazyApplyM(lambda x: x.satisfy                              ())    )
 		if r.ok(): self.lastsat  = 1
 		else:      self.lastfail = 1
 		return r
-	def setup      (self): return self.check(_setup,      lambda: self.lazyApplyU(lambda x: x.setup           ())[0] )
-	def restore    (self): return self.check(_restore,    lambda: self.lazyApplyM(lambda x: x.restore         ())    )
+	def setup      (self): return self.check(_setup,      lambda: self.lazyApplyU(lambda x: x.setup              ())[0] )
+	def restore    (self): return self.check(_restore,    lambda: self.lazyApplyM(lambda x: x.restore            ())    )
 
 #-- Install
 	def uninstallSet (self        ): return self.check (_uninstallSet, lambda: self.lazyApplyM(lambda x: x.uninstallSet      ())    )
