@@ -45,6 +45,10 @@ mtrans = {  'linux-redhat-7.0':'RedHat-7.0',
 	    'linux-debian-3.1':'Debian-3.1',
 	    'linux-debian-3.2':'Debian-3.2',
 	    'linux-debian-3.3':'Debian-3.3',
+	    'linux-SLC-3.0'   :'SLC-3.0',
+	    'linux-SLC-3.1'   :'SLC-3.1',
+	    'linux-SLC-3.2'   :'SLC-3.2',
+	    'linux-SLC-3.3'   :'SLC-3.3',
 	    'linux-debian-3.4':'Debian-3.4' }
 
 machines = []
@@ -80,6 +84,16 @@ machines.append(('RHEL',      'Linux',))
 machines.append(('Debian',    'Linux',))
 machines.append(('linux',     'Linux',))
 machines.append(('Linux',     'linux',))
+machines.append(('Tao',       'Linux',))
+machines.append(('SL',        'Linux',))
+
+#
+#-- Scientific Linux Family
+#
+machines.append(('Fermi-SL',           'SL',))
+machines.append(('SLC',                'SL',))
+machines.append(('CERN-SL',           'SLC',))
+machines.append(('SLC',           'CERN-SL',))
 
 #
 #-- Debian
@@ -108,16 +122,20 @@ machines.append((   'RHEL-1',  'RHEL',      ))
 #
 #-- SuSE linux
 #
-machines.append((    'SuSE',   'suse',  ))
-machines.append((    'suse',   'SuSE',  ))
-machines.append((   'SuSE-8',  'SuSE-7',))
-machines.append((   'SuSE-7',  'SuSE-6',))
-machines.append((   'SuSE-6',  'SuSE-5',))
-machines.append((   'SuSE-5',  'SuSE-4',))
-machines.append((   'SuSE-4',  'SuSE-3',))
-machines.append((   'SuSE-3',  'SuSE-2',))
-machines.append((   'SuSE-2',  'SuSE-1',))
-machines.append((   'SuSE-1',  'SuSE',  ))
+machines.append((   'SuSE',    'suse',    ))
+machines.append((   'suse',    'SuSE',    ))
+machines.append((   'SuSE-9.3','SuSE-9.2',))
+machines.append((   'SuSE-9.2','SuSE-9.1',))
+machines.append((   'SuSE-9.1','SuSE-9',  ))
+machines.append((   'SuSE-9',  'SuSE-8',  ))
+machines.append((   'SuSE-8',  'SuSE-7',  ))
+machines.append((   'SuSE-7',  'SuSE-6',  ))
+machines.append((   'SuSE-6',  'SuSE-5',  ))
+machines.append((   'SuSE-5',  'SuSE-4',  ))
+machines.append((   'SuSE-4',  'SuSE-3',  ))
+machines.append((   'SuSE-3',  'SuSE-2',  ))
+machines.append((   'SuSE-2',  'SuSE-1',  ))
+machines.append((   'SuSE-1',  'SuSE',    ))
 
 #
 #-- Sun
@@ -178,6 +196,16 @@ machines.append((  'CygWin', 'cygwin',))
 machines.append((  'darwin', 'Darwin',))
 machines.append((  'Darwin', 'darwin',))
 
+#
+#-- MacOS
+#
+#machines.append((   'MacOS-X 10.4',  'MacOS-X',  ))
+#machines.append((   'MacOS-X',       'MacOS',    ))
+machines.append((   'MacOS',         'Mac OS',   ))
+machines.append((   'Mac OS',        'MacOS',    ))
+machines.append((   'MacOS',         'Darwin',   ))
+machines.append((  'Darwin',         'MacOS',    ))
+
 #
 #-- misc
 #
@@ -255,8 +283,61 @@ machines.append(( 'linux-fermi-7.2','linux-fermi-7.1',))
 machines.append(( 'linux-fermi-9.0','linux-fermi-9.0',))
 machines.append(( 'linux-fermi-7.1','linux-fermi',    ))
 machines.append(( 'linux-fermi',    'Linux',          ))
+
+#
+#-- Tao Linx
+#
+machines.append(('linux-tao-1',   'TAO-1',        ))
+machines.append(('TAO-1',         'linux-tao-1',  ))
+machines.append(('TAO-1',         'Tao-1',        ))
+machines.append(('Tao-1',         'TAO-1',        ))
+machines.append(('Tao-1',         'Tao',          ))
+
+#
+#-- CERN Scientific Linux 
+#
+machines.append(('SLC-4.4','SLC-4.3',  ))
+machines.append(('SLC-4.3','SLC-4.2',  ))
+machines.append(('SLC-4.2','SLC-4.1',  ))
+machines.append(('SLC-4.1','SLC-4.0',  ))
+machines.append(('SLC-4.0','SLC-4',    ))
+machines.append(('SLC-4'  ,'SLC',      ))
+machines.append(('SLC-3.3','SLC-3.2',  ))
+machines.append(('SLC-3.2','SLC-3.1',  ))
+machines.append(('SLC-3.1','SLC-3.0',  ))
+machines.append(('SLC-3.0','SLC-3',    ))
+machines.append(('SLC-3'  ,'SLC',      ))
+
+machines.append(('SLC-4.4','CERN-SL-4.4',)); machines.append(('CERN-SL-4.4','SLC-4.4',))
+machines.append(('SLC-4.3','CERN-SL-4.3',)); machines.append(('CERN-SL-4.3','SLC-4.3',))
+machines.append(('SLC-4.2','CERN-SL-4.2',)); machines.append(('CERN-SL-4.2','SLC-4.2',))
+machines.append(('SLC-4.1','CERN-SL-4.1',)); machines.append(('CERN-SL-4.1','SLC-4.1',))
+machines.append(('SLC-4.0','CERN-SL-4.0',)); machines.append(('CERN-SL-4.0','SLC-4.0',))
+machines.append(('SLC-4','CERN-SL-4',)); machines.append(('CERN-SL-4','SLC-4',))
+
+machines.append(('SLC-3.4','CERN-SL-3.4',)); machines.append(('CERN-SL-3.4','SLC-3.4',))
+machines.append(('SLC-3.3','CERN-SL-3.3',)); machines.append(('CERN-SL-3.3','SLC-3.3',))
+machines.append(('SLC-3.2','CERN-SL-3.2',)); machines.append(('CERN-SL-3.2','SLC-3.2',))
+machines.append(('SLC-3.1','CERN-SL-3.1',)); machines.append(('CERN-SL-3.1','SLC-3.1',))
+machines.append(('SLC-3.0','CERN-SL-3.0',)); machines.append(('CERN-SL-3.0','SLC-3.0',))
+machines.append(('SLC-3','CERN-SL-3',)); machines.append(('CERN-SL-3','SLC-3',))
+
+#
+#-- Fermi Scientific Linux
+#
+machines.append(('Fermi-SL-4.4','Fermi-SL-4.3',  ))
+machines.append(('Fermi-SL-4.3','Fermi-SL-4.2',  ))
+machines.append(('Fermi-SL-4.2','Fermi-SL-4.1',  ))
+machines.append(('Fermi-SL-4.1','Fermi-SL-4.0',  ))
+machines.append(('Fermi-SL-4.0','Fermi-SL-4',    ))
+machines.append(('Fermi-SL-4'  ,'Fermi-SL',      ))
+machines.append(('Fermi-SL-3.3','Fermi-SL-3.2',  ))
+machines.append(('Fermi-SL-3.2','Fermi-SL-3.1',  ))
+machines.append(('Fermi-SL-3.1','Fermi-SL-3.0',  ))
+machines.append(('Fermi-SL-3.0','Fermi-SL-3',    ))
+machines.append(('Fermi-SL-3'  ,'Fermi-SL',      ))
 		
-supportedPlatforms = ['RedHat','Fedora','Cygwin','RHEL','AIX','SuSE', 'Debian']
+supportedPlatforms = ['RedHat','Fedora','Cygwin','RHEL','AIX','SuSE', 'Debian','Tao','SL','MacOS']
 
 Machines = FinitePreOrder.FinitePreOrder(machines)
 
@@ -269,10 +350,24 @@ def platformstring():
 	return s
 
 def thisPlatform():
-	pre,plat = switchpar('pretend-platform')
-	if not pre:
-		plat = findPlatform()[0]
-	if mtrans.has_key(plat): plat = mtrans[plat]
+	ppl = os.path.join(pac_anchor,pacmanDir,'ppl')
+	if os.path.exists(ppl):
+		try:
+			f = open(ppl,'r')
+			lines = f.readlines()
+			f.close()
+		except:
+			abort("Can't read ["+ppl+"].")
+		
+		if len(lines)>0 and len(lines[0])>0:
+			plat = string.strip(lines[0][:-1])
+		else:
+			abort("File ["+ppl+"] has been corrupted.")
+	else:
+		pre,plat = switchpar('pretend-platform')
+		if not pre:
+			plat = findPlatform()[0]
+		if mtrans.has_key(plat): plat = mtrans[plat]
 	
 	got_one = 0
 	for x,y in machines: 
@@ -291,18 +386,21 @@ def platformDisplay():
 		niceListOut(c)
 		sys.stdout.write(' }')
 		sys.stdout.write('\n')
+	print ' '
+	print '- Supported Architectures -'
 	for sup in supportedPlatforms:
 		l = []
 		for plat in Machines.items():
 			if Machines.le(plat,sup): l.append(plat)
-		print ' '
-		sys.stdout.write('- Supported Architecture ['+sup+']:')
+#		sys.stdout.write('- Supported Architecture ['+sup+']:')
+		sys.stdout.write('      '+sup+':')
 		sort(l,lambda x,y: string.lower(x)<=string.lower(y))
 		sys.stdout.write(' { ')
 		niceListOut(l)
 		sys.stdout.write(' }')
 		sys.stdout.write('\n')
-	print ' '
+		print ' '
+#	print ' '
 	sys.stdout.write('- Your platform ['+p.str()+'] satisfies: ')
 	l = []
 	for plat in Machines.items():
@@ -324,6 +422,16 @@ class Platform(Environment):
 	def str(self): return self._platform
 	def equal(self,x): return self._platform==x._platform
 	
+	def display(self,indent=0):
+		sys.stdout.write(indent*' ')
+		sys.stdout.write('Your platform ['+self._platform+'] satisfies { ')
+		l = []
+		for plat in Machines.items():
+			if Machines.le(self.str(),plat): l.append(plat)
+		sort(l,lambda x,y: string.lower(x)<=string.lower(y))
+		niceListOut(l)
+		sys.stdout.write(' }\n')
+	
 	def platforms(self): return Machines.items()
 	def platformClasses(self):
 		c = Clusters(Machines.items())
